﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairDataModels.Models;
using Microsoft.Extensions.Logging;
using System.Windows.Forms;
using PlumbingRepairContracts.DI;

namespace PlumbingRepair
{
    public partial class FormWork : Form
    {
        private readonly ILogger _logger;

        private readonly IWorkLogic _logic;

        private int? _id;

        private Dictionary<int, (IComponentModel, int)> _workComponents;

        public int Id { set { _id = value; } }
        public FormWork(ILogger<FormWork> logger, IWorkLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
            _workComponents = new Dictionary<int, (IComponentModel, int)>();
        }
        private void FormWork_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка изделия");

                try
                {
                    var view = _logic.ReadElement(new WorkSearchModel
                    {
                        Id = _id.Value
                    });

                    if (view != null)
                    {
                        WorkNameTextBox.Text = view.WorkName;
                        PriceTextBox.Text = view.Price.ToString();
                        _workComponents = view.WorkComponents ?? new Dictionary<int, (IComponentModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки изделия");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка компонент изделия");

            try
            {
                if (_workComponents != null)
                {
                    DataGridView.Rows.Clear();
                    foreach (var pc in _workComponents)
                    {
                        DataGridView.Rows.Add(new object[] { pc.Key, pc.Value.Item1.ComponentName, pc.Value.Item2 });
                    }
                    PriceTextBox.Text = CalcPrice().ToString();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки компонент изделия");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void AddButton_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormWorkComponent>();
                if (form.ShowDialog() == DialogResult.OK)
                {
                if (form.ComponentModel == null)
                {
                    return;
                }

                _logger.LogInformation("Добавление нового компонента: { ComponentName} - { Count}", form.ComponentModel.ComponentName, form.Count);

                if (_workComponents.ContainsKey(form.Id))
                {
                    _workComponents[form.Id] = (form.ComponentModel, form.Count);
                }

                else
                {
                    _workComponents.Add(form.Id, (form.ComponentModel, form.Count));
                }

                LoadData();
            }
        }

        private void ChangeButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
				var form = DependencyManager.Instance.Resolve<FormWorkComponent>();
                int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells[0].Value);
                form.Id = id;
                form.Count = _workComponents[id].Item2;

                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.ComponentModel == null)
                    {
                        return;
                    }

                    _logger.LogInformation("Изменение компонента: { ComponentName} - { Count}", form.ComponentModel.ComponentName, form.Count);
                    _workComponents[form.Id] = (form.ComponentModel, form.Count);
                    LoadData();
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    try
                    {
                        _logger.LogInformation("Удаление компонента: { ComponentName} - { Count}", DataGridView.SelectedRows[0].Cells[1].Value);
                        _workComponents?.Remove(Convert.ToInt32(DataGridView.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }

                    LoadData();
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(WorkNameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(PriceTextBox.Text))
            {
                MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (_workComponents == null || _workComponents.Count == 0)
            {
                MessageBox.Show("Заполните компоненты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Сохранение изделия");

            try
            {
                var model = new WorkBindingModel
                {
                    Id = _id ?? 0,
                    WorkName = WorkNameTextBox.Text,
                    Price = Convert.ToDouble(PriceTextBox.Text),
                    WorkComponents = _workComponents
                };

                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);

                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения изделия");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private double CalcPrice()
        {
            double price = 0;

            foreach (var elem in _workComponents)
            {
                price += ((elem.Value.Item1?.Cost ?? 0) * elem.Value.Item2);
            }

            return Math.Round(price * 1.1, 2);
        }
    }
}
