﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairContracts.StoragesContracts;
using PlumbingRepairContracts.ViewModels;
using Microsoft.Extensions.Logging;
using PlumbingRepairDataModels.Enums;
using PlumbingRepairBusinessLogic.MailWorker;

namespace PlumbingRepairBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
		private readonly AbstractMailWorker _mailWorker;
		private readonly IClientLogic _clientLogic;

		public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker mailWorker, IClientLogic clientLogic)
        {
            _logger = logger;
            _orderStorage = orderStorage;
			_mailWorker = mailWorker;
			_clientLogic = clientLogic;
		}

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }

            model.Status = OrderStatus.Принят;
			var result = _orderStorage.Insert(model);

			if (result == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Insert operation failed");
                return false;
            }

			SendOrderMessage(result.ClientId, $"Ремонт сантехники, Заказ №{result.Id}", $"Заказ №{result.Id} от {result.DateCreate} на сумму {result.Sum:0.00} принят");

			return true;
        }
		public bool StatusUpdate(OrderBindingModel model, OrderStatus newStatus)
		{
			var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
			if (viewModel == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (viewModel.Status + 1 != newStatus)
			{
				_logger.LogWarning("Update operation failed. Order status incorrect.");
				return false;
			}
			model.Status = newStatus;
			model.DateCreate = viewModel.DateCreate;
			model.Sum = viewModel.Sum;
			model.Count = viewModel.Count;
			model.DateImplement = viewModel.DateImplement;
			model.WorkId = viewModel.WorkId;
			if (viewModel.ImplementerId.HasValue)
			{
				model.ImplementerId = viewModel.ImplementerId;
			}
			if (model.Status == OrderStatus.Готов) model.DateImplement = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);
			else
			{
				model.DateImplement = viewModel.DateImplement;
			}
			CheckModel(model);

			var result = _orderStorage.Update(model);
			if (result == null)
			{
				model.Status--;
				_logger.LogWarning("Update operation failed");
				return false;
			}
			SendOrderMessage(result.ClientId, $"Ремонт сантехники, Заказ №{result.Id}", $"Заказ №{model.Id} изменен статус на {result.Status}");
			return true;
		}

		private bool SendOrderMessage(int clientId, string subject, string text)
		{
			var client = _clientLogic.ReadElement(new() { Id = clientId });

			if (client == null)
			{
				return false;
			}

			_mailWorker.MailSendAsync(new()
			{
				MailAddress = client.Email,
				Subject = subject,
				Text = text
			});

			return true;
		}

		public bool TakeOrderInWork(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выполняется);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
			return StatusUpdate(model, OrderStatus.Выдан);
		}

        public bool FinishOrder(OrderBindingModel model)
        {
			return StatusUpdate(model, OrderStatus.Готов);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("Order. OrderId:{Id}", model?.Id);

            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (model.WorkId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор изделия", nameof(model.WorkId));
            }

            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество изделий в заказе должно быть больше 0", nameof(model.Count));
            }

            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }

            _logger.LogInformation("Order. OrderId:{Id}.Sum:{ Sum}. WorkId: { WorkId}", model.Id, model.Sum, model.WorkId);
        }

		public OrderViewModel? ReadElement(OrderSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
			var element = _orderStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}
	}
}
